##############################################################################
#
# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
""" The CCTool modified from CalendarTool """

from Products.CMFCore.utils import UniqueObject
from OFS.ObjectManager import ObjectManager
from OFS.SimpleItem import SimpleItem
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo, getSecurityManager
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import ManagePortal, ModifyPortalContent
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

jurisdictions = {'CC':       {"at": "Austria",
                              "au": "Australia",
                              "be": "Belgium",
                              "br": "Brazil",
                              "ca": "Canada",
                              "de": "Germany",
                              "es": "Spain",
                              "fi": "Finland",
                              "fr": "France",
                              "hr": "Croatia",
                              "it": "Italy",
                              "jp": "Japan",
                              "kr": "Korea",
                              "nl": "Netherlands",
                              "tw": "Taiwan",
                              "uk": "England and Wales"},
                 'Sampling': {"generic": "Generic",
                              "br": "Brazil",}
                }

# srr_icon = Some Right Reserved Icon
licenses =  {"CC":          {'srr_icon': "somerights20.gif",
                             'icon':     "cc.gif",
                             'ccurl':    ['by', '2.0/'],
                             'rights':   [],
                             'lname':    'Creative Commons License'},
             "by":          {'srr_icon': "somerights20.gif",
                             'icon':     "by.gif",
                             'ccurl':    ['by', '2.0/'],
                             'rights':   [],
                             'lname':    'Creative Commons License'},
             "by-nc":       {'srr_icon': "somerights20.gif",
                             'icon':     "nc.gif",
                             'ccurl':    ['by-nc', '2.0/'],
                             'rights':   [],
                             'lname':    'Creative Commons License'},
             "by-nd":       {'srr_icon': "somerights20.gif",
                             'icon':     "nd.gif",
                             'ccurl':    ['by-nd', '2.0/'],
                             'rights':   [],
                             'lname':    'Creative Commons License'},
             "by-sa":       {'srr_icon': "somerights20.gif",
                             'icon':     "sa.gif",
                             'ccurl':    ['by-sa', '2.0/'],
                             'rights':   [],
                             'lname':    'Creative Commons License'},
             "by-nc-nd":    {'srr_icon': "somerights20.gif",
                             'icon':     "cc.gif",
                             'ccurl':    ['by-nc-nd', '2.0/'],
                             'rights':   [],
                             'lname':    'Creative Commons License'},
             "by-nc-sa":    {'srr_icon': "somerights20.gif",
                             'icon':     "cc.gif",
                             'ccurl':    ['by-nc-sa', '2.0/'],
                             'rights':   [],
                             'lname':    'Creative Commons License'},
             "publicdomain":{'srr_icon': "norights-a.gif",
                             'icon':     "40publicdomain.gif",
                             'ccurl':    ['publicdomain', ''],
                             'rights':   ['Reproduction','Distribution','DerivativeWorks'],
                             'lname':    'Public Domain'},
             "devnations":  {'srr_icon': "somerights20.gif",
                             'icon':     "40devnations.gif",
                             'ccurl':    ['devnations', '2.0/'],
                             'rights':   ['Reproduction','Distribution','Notice','Attribution','DerivativeWorks',
                                          'HighIncomeNationUse'],
                             'lname':    'Creative Commons Developing Nations License'},
             "sampling":    {'srr_icon': "sampling.gif",
                             'icon':     "40sampling.gif",
                             'ccurl':    ['sampling', '1.0/'],
                             'rights':   ['Reproduction','Notice','Attribution','Sharing'],
                             'lname':    'Creative Commons License'},
             "sampling+":   {'srr_icon': "sampling.gif",
                             'icon':     "40sampling.gif",
                             'ccurl':    ['sampling+', '1.0/'],
                             'rights':   ['Reproduction','Notice','Attribution'],
                             'lname':    'Creative Commons License'},
             "sampling+nc": {'srr_icon': "sampling.gif",
                             'icon':     "40sampling.gif",
                             'ccurl':    ['nc-sampling+', '1.0/'],
                             'rights':   ['Reproduction','Distribution','Notice','Attribution','CommercialUse'],
                             'lname':    'Creative Commons License'},
             "deed-music":  {'srr_icon': "music.gif",
                             'icon':     "40sharemusic.gif",
                             'ccurl':    ['by-nc-nd/2.0', 'deed-music'],
                             'rights':   ['Reproduction','Distribution','Notice','Attribution'],
                             'lname':    'Creative Commons Music Sharing License'},
             "GPL":         {'srr_icon': "cc-GPL-a.png",
                             'icon':     "40gnugpl.gif",
                             'ccurl':    ['GPL', '2.0/'],
                             'rights':   ['Reproduction','Distribution','Notice','Attribution','DerivativeWorks',
                                          'SourceCode'],
                             'lname':    'CC-GNU GPL'},
             "LGPL":        {'srr_icon': "cc-LGPL-a.png",
                             'icon':     "40gnulgpl.gif",
                             'ccurl':    ['LGPL', '2.1/'],
                             'rights':   ['Reproduction','Distribution','Notice','Attribution','DerivativeWorks'
                                          'SourceCode'],
                             'lname':    'CC-GNU LGPL'}
            }

ccurl = "http://creativecommons.org/licenses/%s/%s"

def addCCTool(self,REQUEST=None):
    """ add CCTool in Plone site. """
    ct = CCTool()
    self._setObject('portal_cc', ct, set_owner=0)
    # add skin
    skinstool = getToolByName(self, 'portal_skins')
    skin_name = 'cc'
    if skin_name not in skinstool.objectIds():
        addDirectoryViews(skinstool, 'skins', plonecc_globals)
    for skinName in skinstool.getSkinSelections():
        path = skinstool.getSkinPath(skinName) 
        path = [i.strip() for i in  path.split(',')]
        if skin_name not in path:
            path.append(skin_name)
        path = ','.join(path)
        skinstool.addSkinSelection(skinName, path)
    if REQUEST:
        return REQUEST.RESPONSE.redirect(REQUEST['HTTP_REFERER'])

class CCLicenseObject(SimpleItem):
    """ class storing information about the Object's license of CC. """

    meta_type = "CCLicense Object"

    manage_options=(
        {'label':'View','action':'manage_ccobject'},
        ) + SimpleItem.manage_options

    security = ClassSecurityInfo()

    security.declareProtected(ManagePortal, 'manage_ccobject')
    manage_ccobject = PageTemplateFile(
        'www/ccobject', globals(),
        __name__='manage_ccobject')
    
    def __init__(self, id, nc='0', nd='0',jd='', fm='', orchoose=''):
        self.id = id
        self.nc = nc
        self.nd = nd
        self.jd = jd
        self.fm = fm
        self.orchoose = orchoose
        self.title = id.replace('--','/')

    security.declarePublic('getCCInfo')
    def getCCInfo(self):
        """ return CC Object info. """
        # compatible reason
        if not hasattr(self, 'orchoose'):
            self.orchoose = ''
        return {'nc':self.nc, 'nd':self.nd, 'jd':self.jd, 'fm':self.fm,
                'orchoose':self.orchoose}

    security.declarePublic('_update')
    def _update(self, nc='0', nd='0', jd='', fm='', orchoose=''):
        ob = self.unrestrictedTraverse(self.title, None)
        sm = getSecurityManager()
        if ob and sm.checkPermission(ModifyPortalContent, ob):
            self.nc = nc
            self.nd = nd
            self.jd = jd
            self.fm = fm
            self.orchoose = orchoose

InitializeClass(CCLicenseObject)

class CCTool (UniqueObject, ObjectManager, SimpleItem):
    """ a license tool for Creative Commons. """
    id = 'portal_cc'
    meta_type= 'Creative Commons Tool'
    security = ClassSecurityInfo()
    plone_tool = 1

    manage_options = ( ({ 'label' : 'Overview', 'action' : 'manage_overview' }
                     ,  { 'label' : 'Configure', 'action' : 'manage_configure' }
                     ,
                     ) + ObjectManager.manage_options
                     )

    security.declareProtected( 'View', 'manage_overview' )
    manage_overview = PageTemplateFile('www/explainCCTool', globals(),
                                   __name__='manage_overview')

    security.declareProtected( ManagePortal, 'manage_configure' )
    manage_configure = PageTemplateFile('www/configureCCTool', globals(),
                                   __name__='manage_configure')

    def __init__(self):
        self.cc_types = ['Document','Image','File','News Item']
        self._db = {}

    security.declareProtected( ManagePortal, 'edit_configuration' )
    def edit_configuration(self, show_types):
        """ Change the configuration of the cc tool """
        self.cc_types = show_types
        if hasattr(self.REQUEST, 'form_url'):
            self.REQUEST.RESPONSE.redirect(self.REQUEST.form_url+'?portal_status_message=Configuration changed!')

    security.declarePublic('getCCTypes')
    def getCCTypes(self):
        """ Returns a list of type that will show in the calendar """
        return self.cc_types

    security.declarePublic('getJurisdictions')
    def getJurisdictions(self,license):
        """ Returns a list of jurisdictions that will governed cc license. """
        return jurisdictions[license].keys()

    security.declarePublic('getJurisdictionsName')
    def getJurisdictionsName(self, license):
        """ Returns a dictionary of jurisdictions name that will governed cc license. """
        return jurisdictions[license]

    security.declarePublic('openCCHelpWindow')
    def openCCHelpWindow(self, helpname):
        """ Returns a javascript code to open help window. """
        javascriptCode = "window.open('/%s', '%s', 'width=375,height=300,scrollbars=yes,resizable=yes,toolbar=no,directories=no,location=yes,menubar=no,status=yes');return false;" % (helpname, helpname)
        return javascriptCode

    security.declarePublic('getCCObject')
    def getCCObject(self, objectpath, deflicense=None):
        """ Returns cc object. """
        cco = self._getOb(objectpath, None)
        if cco:
            return cco.getCCInfo()
        elif deflicense:
            cco = {'nc': '', 'nd': '', 'jd': '', 'fm': '', 'orchoose': ''}
            if deflicense[:2] == 'by':
                if 'nc' in deflicense:
                    cco['nc']='1'
                if 'nd' in deflicense:
                    cco['nd']='2'
                if 'sa' in deflicense:
                    cco['nd']='1'
            else:
                cco['orchoose']=deflicense
            return cco
        else:
            return None

    security.declarePublic('getLicenseInfo')
    def getLicenseInfo(self, objectpath='', infokey=None, deflicense=None):
        """ Returns license information. """
        if infokey == None:
            # return ALL licenses name.
            return licenses.keys()
        cco = self._getOb(objectpath, None)
        if cco:
            CCO = cco.getCCInfo()
        elif deflicense:
            return licenses[deflicense][infokey]
        orchoose = CCO['orchoose']
        if orchoose != '' and infokey in licenses[orchoose].keys():
            return licenses[orchoose][infokey]
        elif infokey in licenses['CC'].keys():
            return licenses['CC'][infokey]

    security.declarePublic('setCCObject')
    def setCCObject(self, objectpath, NonCommercial='', NonDerivative='', 
                    jurisdiction='', format='', orchoose=''):
        """ Change the configuration of the cc tool """
        cco = self._getOb(objectpath, None)
        ob = self.unrestrictedTraverse(objectpath.replace('--','/'), None)
        sm = getSecurityManager()
        if ob and sm.checkPermission(ModifyPortalContent, ob):
            if cco:
                cco._update(NonCommercial,NonDerivative,jurisdiction,format,orchoose)
                return 'License changed!'
            else:
                cco = CCLicenseObject(objectpath, nc=NonCommercial, nd=NonDerivative, 
                        jd=jurisdiction, fm=format, orchoose=orchoose)
                self._setObject(objectpath, cco)
                return 'License assigned!'

    security.declarePublic('delCCObject')
    def delCCObject(self, objectpath):
        """ Delete the CCObject from the cc tool """
        ob = self.unrestrictedTraverse(objectpath.replace('--','/'), None)
        sm = getSecurityManager()
        if ob and sm.checkPermission(ModifyPortalContent, ob):
            if self._getOb(objectpath, None):
                self._delObject(objectpath)
                return 'License deleted!'
            else:
                return 'License not assigned!'

    security.declarePublic('isCCType')
    def isCCType(self, contextType):
        """ Returns 1 when context belong to cc types, else None. """
        if contextType in self.getCCTypes():
            return True
        else:
            return False

    security.declarePublic('isCCObject')
    def isCCObject(self, objectpath):
        """ Returns true when object path was matched a CCLicenseObject. """
        if self._getOb(objectpath, None):
            return True
        else:
            return False

    security.declarePublic('getCCLicenseLink')
    def getCCLicenseLink(self, objectpath, deflicense=None):
        """ Returns object's CC License link when object path was matched a CCLicenseObject. """
        cco = self._getOb(objectpath, None)
        if cco:
            CCObject = cco.getCCInfo()
            # Return orchoose if that are not empty string.
            orchoose = CCObject['orchoose']
            if orchoose != '':
                return ccurl % tuple(licenses[orchoose]['ccurl'])
            cclink = licenses['CC']['ccurl'][:]
            # nc == '1' => NonCommercial
            if CCObject['nc'] == '1':
                cclink[0] += '-nc'
            # nd == '1' => Share Alike
            # nd == '2' => Non Derivative
            if CCObject['nd'] == '1':
                cclink[0] += '-sa'
            elif CCObject['nd'] == '2':
                cclink[0] += '-nd'
        elif deflicense:
            cclink = licenses[deflicense]['ccurl'][:]
        return ccurl % tuple(cclink)

InitializeClass(CCTool)
